"use strict";
var StateChangerManager = (function () {
    function StateChangerManager() {
        this.stateChangers = {};
    }
    StateChangerManager.prototype.registerStateChanger = function (stateChanger, currentState) {
        if (this.stateChangers[stateChanger.name]) {
            throw new Error("StateChanger with name '" + stateChanger.name + "' already registered!");
        }
        this.stateChangers[stateChanger.name] = stateChanger;
        for (var key in stateChanger.initialState) {
            var namespacedKey = stateChanger.name + "." + key;
            //Got no Initial State on initialization
            if (!currentState[namespacedKey]) {
                currentState[namespacedKey] = stateChanger.initialState[key];
            }
        }
        return currentState;
    };
    StateChangerManager.prototype.unregisterStateChanger = function (stateChanger) {
        if (!this.stateChangers[stateChanger.name]) {
            throw new Error("StateChanger with name '" + stateChanger.name + "' not registered!");
        }
        delete this.stateChangers[stateChanger.name];
    };
    StateChangerManager.prototype.checkStateKey = function (currentState, stateKey, stateChangerName) {
        if (currentState[stateChangerName + "." + stateKey] == undefined) {
            throw new Error("No state property '" + stateChangerName + "." + stateKey + "' registered!");
        }
    };
    Object.defineProperty(StateChangerManager.prototype, "registeredStateChangers", {
        get: function () {
            var stateChangerNames = [];
            for (var stateChanger in this.stateChangers) {
                stateChangerNames.push(stateChanger);
            }
            return stateChangerNames;
        },
        enumerable: true,
        configurable: true
    });
    return StateChangerManager;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = StateChangerManager;
